# Copyright 2008 Canonical Ltd.  All rights reserved.
"""Tests for ETag generation."""

import unittest

from lazr.restful._resource import make_entry_etag_cores

class TestEntryResourceETags(unittest.TestCase):
    # The EntryResource uses the field values that can be written or might
    # othwerise change as the basis for its ETags.  The make_entry_etag_cores
    # function is passed the data about the fields and returns the read and
    # write cores.

    def test_no_field_details(self):
        # If make_entry_etag_cores is given no field details (because no
        # fields exist), the resulting cores empty strings.
        self.assertEquals(make_entry_etag_cores([]), ['', ''])

    def test_writable_fields(self):
        # If there are writable fields, their values are incorporated into the
        # writable portion of the cores.
        field_details = [
            ('first_field',
             {'writable': True,
              'modifiable': False,
              'value': 'first'}),
            ('second_field',
             {'writable': True,
              'modifiable': False,
              'value': 'second'}),
            ]
        self.assertEquals(
            make_entry_etag_cores(field_details), ['', 'first\0second'])

    def test_modifiable_fields(self):
        # If there are modifiable fields -- fields that aren't writable, but
        # might change nonetheless -- their values are incorporated into the
        # "readable" portion of the cores.
        field_details = [
            ('first_field',
             {'writable': False,
              'modifiable': True,
              'value': 'first'}),
            ('second_field',
             {'writable': False,
              'modifiable': True,
              'value': 'second'}),
            ]
        self.assertEquals(
            make_entry_etag_cores(field_details), ['first\0second', ''])

    def test_unchanging_fields(self):
        # If there are fields that are not modifiable their values are not
        # reflected in the generated cores.
        field_details = [
            ('first_field',
             {'writable': False,
              'modifiable': False,
              'value': 'the value'}),
            ]
        self.assertEquals(
            make_entry_etag_cores(field_details),
            ['', ''])

    def test_combinations_of_fields(self):
        # If there are a combination of writable, changable, and unchanable
        # fields, their values are -- or are not in the case of unchanable
        # fields -- reflected in the resulting cores.
        field_details = [
            ('first_writable',
             {'writable': True,
              'modifiable': False,
              'value': 'first-writable'}),
            ('second_modifiable',
             {'writable': True,
              'modifiable': False,
              'value': 'second-writable'}),
            ('first_modifiable',
             {'writable': False,
              'modifiable': True,
              'value': 'first-modifiable'}),
            ('second_modifiable',
             {'writable': False,
              'modifiable': True,
              'value': 'second-modifiable'}),
            ('first_unchanging',
             {'writable': False,
              'modifiable': False,
              'value': 'first-unchanging'}),
            ('second_unchanging',
             {'writable': False,
              'modifiable': False,
              'value': 'second-unchanging'}),
            ]
        self.assertEquals(
            make_entry_etag_cores(field_details),
            ['first-modifiable\0second-modifiable',
             'first-writable\0second-writable'])
